@@ -10,7 +10,7 @@ use Scalar::Util qw(blessed reftype weaken);
 use Socket qw(inet_aton AF_INET);
 use Time::HiRes qw(gettimeofday tv_interval);
 
-our $VERSION = '0.009';
+our $VERSION = '0.010';
 
 my $DEFAULT_FORMAT = 'common';
 my %FORMATS = (
@@ -33,8 +33,8 @@ sub register {
     my ($self, $app, $conf) = @_;
     my $log = $conf->{log} // $app->log->handle;
     my ($pkg, $f, $l) = caller 2;   # :-/
-    my $fh;
 
+    $app->log->warn(__PACKAGE__ . '::VERSION = ' . $VERSION);
     unless ($log) { # somebody cleared $app->log->handle?
         # Log a warning nevertheless - there might be an event handler.
         $app->log->warn(__PACKAGE__ . ': Log handle is not defined');
@@ -45,9 +45,8 @@ sub register {
     my $logger;
 
     if ($reftype eq 'GLOB') {
-        $fh = $log;
-        eval { $fh->autoflush(1) };
-        $logger = sub { $fh->print($_[0]) };
+        eval { $log->autoflush(1) };
+        $logger = sub { $log->print($_[0]) };
     }
     elsif (blessed($log) and my $l = $log->can('print') || $log->can('info')) {
         $logger = sub { $l->($log, $_[0]) };
@@ -59,7 +58,7 @@ sub register {
         File::Spec->file_name_is_absolute($log)
             or $log = $app->home->rel_file($log);
 
-        $fh = IO::File->new($log, '>>')
+        my $fh = IO::File->new($log, '>>')
             or die <<"";
 Can't open log file "$log": $! at $f line $l.
 
@@ -112,6 +111,7 @@ uname_helper is DEPRECATED in favor of \$c->req->env->{REMOTE_USER} at $f line $
     # each handler is called with following parameters:
     # 0: $tx, 1: $tx->req, 2: $tx->res, 3: $tx->req->url,
     # 4: $request_start_time, 5: $process_time, 6: $bytes_in, 7: $bytes_out
+    # 8: HTTP request start line
 
     my $block_handler = sub {
         my ($block, $type) = @_;
@@ -173,7 +173,7 @@ uname_helper is DEPRECATED in favor of \$c->req->env->{REMOTE_USER} at $f line $
             my $s = $_[3]->query->to_string or return '';
             return '?' . $s;
         },
-        r => sub { substr($_[1]->build_start_line, 0, -2) },
+        r => sub { $_[8] },
         s => sub { $_[2]->code // '-' },
         t => sub {
             $strftime->('[%d/%b/%Y:%H:%M:%S %z]', localtime($_[4][0]))
@@ -227,28 +227,32 @@ uname_helper is DEPRECATED in favor of \$c->req->env->{REMOTE_USER} at $f line $
 
     $app->hook(after_build_tx => sub {
         my $tx = $_[0];
-        my $bcr = my $bcw = 0;
-        my ($r, $s, $t, $w);
 
         $tx->on(connection => sub {
             my ($tx, $connection) = @_;
+            my $bcr = my $bcw = 0;
+            my $sl;
+            my $t = [gettimeofday];
+            my $s = Mojo::IOLoop->stream($connection);
+            my $r = $s->on(read  => sub {
+                # get the unmodified HTTP request start line
+                $sl //= substr($_[1], 0, index($_[1], "\r\n"));
+                $bcr += length $_[1];
+            });
+            my $w = $s->on(write => sub { $bcw += length $_[1] });
 
-            $t = [gettimeofday];
-            $s = Mojo::IOLoop->stream($connection);
-            $r = $s->on(read  => sub { $bcr += length $_[1] });
-            $w = $s->on(write => sub { $bcw += length $_[1] });
-        });
+            weaken $s;
+            weaken $r;
+            weaken $w;
 
-        # watch for the right moment to fetch the un-expanded start-line
-        $tx->req->once(progress => sub { $_[0]->build_start_line });
+            $tx->on(finish => sub {
+                my $tx = shift;
+                my $dt = tv_interval($t);
 
-        $tx->on(finish => sub {
-            my $tx = shift;
-            my $dt = tv_interval($t);
-
-            $s->unsubscribe(read  => $r);
-            $s->unsubscribe(write => $w);
-            $logger->(_log($tx, $format, \@handler, $t, $dt, $bcr, $bcw));
+                $s->unsubscribe(read  => $r);
+                $s->unsubscribe(write => $w);
+                $logger->(_log($tx, $format, \@handler, $t, $dt, $bcr, $bcw, $sl));
+            });
         });
     });
 }
@@ -281,7 +285,7 @@ Mojolicious::Plugin::AccessLog - An AccessLog Plugin for Mojolicious
 
 =head1 VERSION
 
-Version 0.009
+Version 0.010
 
 =head1 SYNOPSIS
 
